// ======================================================================
//
// geometry.h
// jeff grills
//
// copyright 1998 origin systems inc
//
// ======================================================================

#ifndef GEOMETRY_H
#define GEOMETRY_H

// ======================================================================

class GL_abstract;

#include "Camera.h"
#include "Xform.h"
#include "Vector.h"
#include "Gl.def"

// ======================================================================

class Geometry
{
public:

	enum
	{
		CLIPPED_BUFFER                 = 16,
		TVERTEX_BUFFER                 = 256 + 16,
		TVERTEX_BUFFER_SAFE            = TVERTEX_BUFFER - 16
	};

private:

	// graphics layer
	static GL_abstract  *gl;

	// flags
	static bool         needObjectToCameraRebuilt;
	static bool         needLightingRecalculated;

	// frustum variables
	static real         nearPlane, farPlane;
	static real         clipWidthMultiplier, clipHeightMultiplier;
	static Vector       frustumNormal[6];
	static real         frustumPlaneD[6];
	
	// transforms 
	static Xform        objectToWorld;
	static Xform        worldToCamera;
	static Xform        objectToCamera;

	// light information
	static bool         lightingEnabled;
	static int          lightCount;
	static Light      **lightList;
	static int          applyLightCount;
	static int          applyLightSize;
	static Light      **applyLightList;

	// model information
	static const Vector *vertex;
	static int           vertexCount;
	static const Vector *normal;
	static int           normalCount;

	// transform buffers
	static int           viewspaceVertexSize;
	static Vector       *viewspaceVertex;
	static int8         *viewspaceVertexClipFlags;
	static TVertex       transformedVertexBuffer[TVERTEX_BUFFER];
	static TVertex       clippedVertexBuffer[CLIPPED_BUFFER];

private:

	static void rebuildTransforms(void);
	static void rebuildTransformsReal(void);

	static void recalculateLighting(void);
	static void recalculateLightingReal(void);

public:

	static void install(GL_abstract *gl);
	static void remove(void);

	// 3d transformation functions
	static void setCamera(const Camera &camera);
	static void setObjectToWorld(const Xform &newObjectToWorld);

	// 3d setup functions
	static void setLightingEnabled(bool newLightingEnabled);
	static void setLightList(Light **newLightList, int newLightCount);
	static bool setVertexList(const Vector *vertex, int vertexCount, const Vector *normal, int normalCount);

	// rendering primitive functions
	static void drawPointList(const UVertex *vertexList, int numberOfVertices);
	static void drawLineList(const UVertex *vertexList, int numberOfVertices);
	static void drawLineStrip(const UVertex *vertexList, int numberOfVertices);
	static void drawTriangleList(const UVertex *vertexList, int numberOfVertices);
	static void drawTriangleStrip(const UVertex *vertexList, int numberOfVertices);
	static void drawTriangleFan(const UVertex *vertexList, int numberOfVertices);
};

// ======================================================================

inline void Geometry::setObjectToWorld(const Xform &newObjectToWorld)
{
	objectToWorld = newObjectToWorld;
	needObjectToCameraRebuilt = true;
	needLightingRecalculated = true;
}

// ----------------------------------------------------------------------

inline void Geometry::rebuildTransforms(void)
{
	if (needObjectToCameraRebuilt)
		rebuildTransformsReal();
}

// ----------------------------------------------------------------------

inline void Geometry::recalculateLighting(void)
{
	if (needLightingRecalculated)
		recalculateLightingReal();
}

// ======================================================================

#endif
